@@ -17,12 +17,18 @@ package options
 import "github.com/spf13/cobra"
 
 type TreeOptions struct {
-	Registry  RegistryOptions
-	CleanType string
+	Registry             RegistryOptions
+	RegistryExperimental RegistryExperimentalOptions
+	CleanType            string
+	ExperimentalOCI11    bool
 }
 
 var _ Interface = (*TreeOptions)(nil)
 
 func (c *TreeOptions) AddFlags(cmd *cobra.Command) {
 	c.Registry.AddFlags(cmd)
+	c.RegistryExperimental.AddFlags(cmd)
+
+	cmd.Flags().BoolVar(&c.ExperimentalOCI11, "experimental-oci11", false,
+		"set to true to enable experimental OCI 1.1 behaviour")
 }
